package cardreader;

use strict;
use vars qw($VERSION @ISA @EXPORT_OK);
 
require Exporter;
require DynaLoader;

@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
$VERSION = '0.90';
@EXPORT_OK = qw(
	ISO7816Error
);

bootstrap cardreader $VERSION;

# Preloaded methods go here.

sub new
{
	my $class = shift;
	my $type_txt = shift;
	my $device = shift;
	my $desc = {};
	my $ret;
	my $type = 0;

	bless $desc, $class;

	# defalt type
	if (defined $type_txt)
	{
		$type = 0 if ($type_txt =~ m/^TLP$/i);
		$type = 1 if ($type_txt =~ m/^RFTLP$/i);
	}

	# default device
	if (! defined $device)
	{
		$device = "/dev/screader";
		$device = "COM1" if ($^O eq "MSWin32");
	}

	$ret = _InitReader($type, $device);
	return (undef) if (! defined $ret);

	$desc -> {desc} = $ret;

	return $desc;
}

sub ResetCard
{
	my $self = shift;
	return _ResetCard($self -> {desc});
}

sub SendIncomingAPDU
{
	my $self = shift;
	my $cmd = shift;
	return _SendIncomingAPDU($self -> {desc}, $cmd);
}

sub SendOutgoingAPDU
{
	my $self = shift;
	my $cmd = shift;
	return _SendOutgoingAPDU($self -> {desc}, $cmd);
}

sub SendIncomingOutgoingAPDU
{
	my $self = shift;
	my $cmd = shift;
	return _SendIncomingOutgoingAPDU($self -> {desc}, $cmd);
}

sub PowerDown
{
	my $self = shift;
	my $ret;

	_PowerDown($self -> {desc}) or return 0;

	return 1;
}

sub ISO7816_ErrorStr
{
	my $self = shift;
	my $code = shift;

	if (length($code) != 4)
	{
		return "Wrong parameter size. Should be 2 bytes (4 hexa chars).";
	}

	return _ISO7816_ErrorStr($code)
}

# Autoload methods go after =cut, and are processed by the autosplit program.

#
# this method is automatically called when the module is loaded
# it is used to load the associated low level SCreader library
#
if (_AllocReaderLib())
{
	# OK, the .so is loaded
	1;
}
else
{
	# problem when loding the .so
	0;
}

__END__
# Below is the stub of documentation for your module. You better edit it!


=head1 NAME

cardreader - Perl extension for TLP and RFTLP SmartCard reader

=head1 SYNOPSIS

use cardreader;

=head1 DESCRIPTION



=head2 new

$desc = new(); or $desc = new($type); or $desc = new($type, $device);

Create a new SmartCard reader object.  If not present $type is set to
"TLP".  If not present $device is set to "COM1" if $OSNAME equals
"MSWin32" and "/dev/screader" otherwise.


=head2 ResetCard

$atr = $desc -> ResetCard;

Sends a reset to the card. The ATR is returned as a string.
An empty string is returned if an error occurs.


=head2 SendIncomingAPDU

$sw = $desc -> SendIncomingAPDU($apdu);

Sends an APDU command to the card.
$apdu is the APDU command as a string.

Return value: the status word as a string ("9000" for no error).


=head2 SendOutgoingAPDU

($res, $sw) = $desc -> SendOutgoingAPDU($apdu);

Sends an outgoing APDU command to the card.
$apdu is the APDU command as a string.

Return Value: a list of 2 strings. The first one is the data,
and the second is the status word (ie ("??????","9000")).


=head2 SendIncomingOutgoingAPDU

($res, $sw) = $desc -> SendIncomingOutgoingAPDU($apdu);

Sends an incoming command to the card, and data are returned by the card.
$apdu is the APDU command as a string.

Return Value: a list of 2 strings. The first one is the data, 
and the second is the status word (ie "??????","9000").


=head2 PowerDown

$desc -> PowerDown;

Sends a power down to the card.


=head2 ISO7816_ErrorStr

$error = $desc -> ISO7816_ErrorStr($sw)

Returns a text string containing the (error) message corresponding to
$sw as defined by ISO 7816-4. All messages are not errors. For example a
Status Word of "9000" corresonds to a "Normal processing".


=head2 Dereferencing

undef $desc;

Close the serial port.

The dereferencing is done automatically when
$desc is allocated a new value or perl(1) exits.


=head1 ERROR CODES

$! can be one of the following values:

=over

=item -6

SC_INVALID_TYPE

=item -5

SC_BAD_HANDLE

=item -4

SC_OPEN_DEVICE

=item -3

SC_NOT_ENOUGH_MEMORY

=item -2

SC_LOAD_FUNCTION

=item -1

SC_LOAD_LIBRARY

=item 0

RD_NO_ERROR

=item 1

RD_UNKNOWN_COMMAND

=item 2

RD_DRIVER_LIMITATION

=item 3

RD_WRONG_ARG_LENGTH

=item 4

RD_PROTOCOL_ERROR

=item 5

RD_WRONG_ATR

=item 6

RD_WRONG_RD_INS

=item 7

RD_MSG_TOO_LONG

=item 8

RD_READ_ERROR

=item 9

RD_CARD_OFF

=item 10

RD_PROG_VOLT_ERROR

=item 11

RD_UNKNOWN_PROTOCOL

=item 12

RD_LENGTH_ERROR

=item 13

RD_WRONG_NB_OF_PARAM

=item 14

RD_UNKNOWN_CARD_PROTOCOL

=item 15

RD_CARD_MUTE

=item 16

RD_CARD_PARITY_ERROR

=item 17

RD_CARD_WRONG_PROCEDURE_BYTE

=item 18

RD_TRANSFERT_TO_CARD_INTERRUPTED

=item 19

RD_CARD_REMOVED

=item 20

RD_SHORT_CIRCUIT

=item 21

RD_CARD_ABSENT

=item 22

RD_READ_FAILED (read on serial port failed)

=item 23

RD_TRANS_ERROR (reader complained about a transmission error)

=item 24

RD_WRITE_FAILED (write to serial port failed)

=item 25

RD_RESP_TOO_LONG (Response form reader exceeded buffer length)

=item 26

RD_READER_MUTE

=item 27

RD_STATUS_TOO_LONG

=item 28

RD_USELESS_ERROR

=item 29

RD_CHAINING_ERROR

=item 30

RD_RESP_TRUNCATED

=item 31

RD_UNKNOWN_ERROR


=back


=head1 AUTHORS

Laurent Gauteron, Gemplus (Laurent.Gauteron@gemplus.com)

Ludovic Rousseau, Gemlpus (Ludovic.Rousseau@gemplus.com)

=head1 SEE ALSO

perl(1p)

=cut

